@@ -215,7 +215,7 @@ xf86OSPMOpen(void)
     }
     xf86PMGetEventFromOs = sunPMGetEventFromOS;
     xf86PMConfirmEventToOs = sunPMConfirmEventToOs;
-    APMihPtr = xf86AddInputHandler(fd, xf86HandlePMEvents, NULL);
+    APMihPtr = xf86AddGeneralHandler(fd, xf86HandlePMEvents, NULL);
     return sunCloseAPM;
 }
 
@@ -225,7 +225,7 @@ sunCloseAPM(void)
     int fd;
 
     if (APMihPtr) {
-	fd = xf86RemoveInputHandler(APMihPtr);
+	fd = xf86RemoveGeneralHandler(APMihPtr);
 	close(fd);
 	APMihPtr = NULL;
     }
